package cz.cvut.fel.pr1;

import javax.imageio.ImageIO;
import java.awt.Color;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.Arrays;

public class ArrayBackedCanvas implements Canvas {

    private final int width, height;
    private final Color[][] canvas;

    /**
     *
     * @param width
     * @param height
     */
    public ArrayBackedCanvas(int width, int height) {
        this.width = width;
        this.height = height;
        canvas = new Color[width][height];
        clearCanvas(Color.WHITE);
    }

    /**
     *
     * @param color
     */
    public void clearCanvas(Color color) {
        for (Color[] row : canvas) {
            Arrays.fill(row, color);
        }
    }

    @Override
    public void setColorAt(int x, int y, Color color) {
        canvas[x][y] = color;
    }

    /**
     * Writes canvas into file. The file format is png.
     *
     * @param fileName target file name
     * @throws IOException
     */
    public void writeToFile(String fileName) throws IOException {
        File file = new File(fileName);
        BufferedImage bufferedImage = generateBufferedImage();

        ImageIO.write(bufferedImage, "png", file);
    }

    private BufferedImage generateBufferedImage() {
        BufferedImage bufferedImage = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);

        for (int x = 0; x < canvas.length; x++) {
            Color[] row = canvas[x];
            for (int y = 0; y < row.length; y++) {
                Color color = row[y];
                java.awt.Color awtColor = new java.awt.Color(color.getRed(), color.getGreen(), color.getBlue());
                bufferedImage.setRGB(x, height - y - 1, awtColor.getRGB());
            }
        }
        return bufferedImage;
    }

    @Override
    public int getWidth() {
        return width;
    }

    @Override
    public int getHeight() {
        return height;
    }

}
